clear all
close all
clc

set(0,'defaultAxesFontName', 'Times New Roman',...
    'defaultTextFontName', 'Times New Roman',...
    'defaultAxesFontSize', 12,...
    'defaultTextFontSize', 12,...
    'defaultLineLineWidth',2,...
    'DefaultAxesXGrid','on',...
    'DefaultAxesYGrid','on',...
    'DefaultTextInterpreter','Tex')
close all
h = 0.01;
%load('NEDC_new')
load('WLTP_low')
figure()
plot(T_z,V_z)
xlabel('Time [s]'), ylabel('Vehicle speed [m/s]')
grid minor


% %%
% filename = 'testdata1.xlsx';
% B=V_z;
% xlswrite(filename,B)
%%

g = 9.81;         % Gravitation constant
rho = 1.21;  % Mass density of the air, [kg/m3]
Time = max(T_z);   %Simulation time 1h, [s]

%%% VEHICLE
M_veh = 3000;   % Vehicle mass, [kg]
Cx = 0.316;  % Aerodynamic drag coefficient
Cz = 0.03;   % Aerodynamic lift coefficient
CMy = 0;   % Pitching moment coefficient 

L = 3.3;  %Wheelbase, [m]
a=925/2155*L; %Horizontal distance from CG to front axle, [m]
b=1230/2155*L; %Horizontal distance from CG to rear axle, [m] 
hG=1.2; %Height of vehicle CG above the ground, [m]

A_f =1.996*2.7*0.85;  % Frontal area of the vehcile, [m2] 0.607


%% WHEEL 
%225/65R16

R_wheel = (16*25.4/2+0.65*225)/1000;   % wheel radius [m]
J_wheel = 0.32; %Moment of inertia [kgm2]
f_r = 0.01;   %Rolling resistance coefficient


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
M = M_veh;


%% INTERNAL COMBUSTION ENGINE 2.3l Diesel engine

load('Engine_map.mat');

Q_ice_map = Q_ice_map/(1.2511*1.35); %/1.251 1.65
% T_ice_resist = [0 5/100*T_ice_max];
% w_ice_resist = [0 w_ice_map];
J_ice = 0.05; % Engine moment of inertia [kgm2]
J_dmfw = 0.323; % Dual mass flywheel moment of inertia [kgm2]
w_CE_idle = min(w_ice_map)*pi/30+5 % [rad/s] idling speed
P_aux = 0;  % Enigne auxilary power requirement

H_gasoline = 4.4e7;                                                        %[J/kg] Low heating power of the gasoline
Vol = 2300/1e6;                                                                       %[cm3]  Volume
i = 2;                                                                              %[-]    4 stroke engine
T2pme = i*2*pi/(Vol)/1e5;                                                      %[bar/Nm]    Torque 2 pme multiply factor 
rho_gasoline = 0.739;                                                      %[kg/dm3] Gasoline density
CV2W = 735.49875;                                                        %[CV/W] Conversion between CV and W
pme2T = (Vol*1e5)/(i*2*pi);                                                      %[bar/Nm]    Torque 2 pme multiply factor 

for i=1:10
    for j=1:200
   Q_ice_map(i,j)=0;
end
end


%% TRANSMISSION

eff_gb = 0.94;  % Efficiency of the gearbox
eff_pulley = 0.90;  % Efficiency of the pulley belt transmission 0.9:0.02:0.98


UI = 4.71; % First gear ratio
UII = 3.14; % Second gear ratio
UIII = 2.11; % Third gear ratio
UIV = 1.67; % Fourth gear ratio
UV = 1.28; % Fifth gear ratio
UVI = 1; % Sixth gear ratio
UVII= 0.84; % Seventh gear ratio
UVIII = 0.67; % Eighth gear ratio
Uf=3.615; % Final gear ratio
Upulley = 2.7;
% Gear shift strategy
Pmax = max( pi/30*w_ice_map.*T_ice_max/T2pme/1e3); % [kW] Max power of the engine

%  n_max_acc_1 = (0.5753*exp(-1.9*Pmax/(M+75)) - 0.1)*(max(w_ice_map)-w_CE_idle*30/pi)+w_CE_idle*30/pi
% n_max_acc_i = (0.5753*exp(-1.9*Pmax/(M+75)))*(max(w_ice_map)-w_CE_idle*30/pi)+w_CE_idle*30/pi

n_max_acc_1 = 2500;
n_min_acc = 1200;

V1_2 = n_max_acc_1/UI/Uf*pi/30*R_wheel; % Gearshift from 1->2
V2_3 = n_max_acc_1/UII/Uf*pi/30*R_wheel; % Gearshift from 2->3
V3_4 = n_max_acc_1/UIII/Uf*pi/30*R_wheel; % Gearshift from 3->4
V4_5 = n_max_acc_1/UIV/Uf*pi/30*R_wheel; % Gearshift from 1->5
V5_6 = n_max_acc_1/UV/Uf*pi/30*R_wheel; % Gearshift from 5->6
V6_7 = n_max_acc_1/UVI/Uf*pi/30*R_wheel; % Gearshift from 6->7
V7_8 = n_max_acc_1/UVII/Uf*pi/30*R_wheel; % Gearshift from 7->8

% n_min_acc = n_max_acc_i*UIV/UIII 

V2_1 = n_min_acc/UII/Uf*pi/30*R_wheel; % Gearshift from 1->2
V3_2 = n_min_acc/UIII/Uf*pi/30*R_wheel; % Gearshift from 2->3
V4_3 = n_min_acc/UIV/Uf*pi/30*R_wheel; % Gearshift from 3->4
V5_4 = n_min_acc/UV/Uf*pi/30*R_wheel; % Gearshift from 1->5
V6_5 = n_min_acc/UVI/Uf*pi/30*R_wheel; % Gearshift from 5->6
V7_6 = n_min_acc/UVII/Uf*pi/30*R_wheel; % Gearshift from 6->7
V8_7 = n_min_acc/UVIII/Uf*pi/30*R_wheel; % Gearshift from 7->8

G_up = [1 2 3 4 5 6 7 8];
V_g_up = 3.6*[V1_2 V2_3 V3_4 V4_5 V5_6 V6_7 V7_8 190/3.6];
G_down = [2 3 4 5 6 7 8];
V_g_down = 3.6*[V2_1 V3_2 V4_3 V5_4 V6_5 V7_6 V8_7];
plot(V_g_up, G_up, 'ro', V_g_down,  G_down, 'bd')
legend 'Upshift' 'Downshift'
grid on
grid minor

%% ELECTRIC MOTOR



load('Motor_map.mat');

J_motor = 0.008; %  Motor shaft inertia [kgm2]

% figure(1), contour(w_EM_max, T_EM_max, eff_motor,[0:0.02:1],'ShowText','on')
% hold on, contour(w_EM_max, T_GEN_max, eff_gen,[0:0.02:1],'ShowText','on')
% grid minor
% xlabel('Angular speed [rpm]'), ylabel('Torque [Nm]')
% xlim([0 8000])
% ylim([-400 400])


%% ELECTRIC BATTERY
%               Nominal Cell Voltage: 3.6V
%				Total Cells: 14s2p (14 cells series 2 parellel) 
%				Nominal Voltage: 48 V 
%				Published Capacity: 8.8 Ah
eb_soc=[0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1];  % (--)
eb_coulombic_eff= .905; 
eb_num_module=7.5; %num of series cells
eb_r_dis=[	0.0377	0.0338	0.0300	0.0280	0.0275	0.0268	0.0269	0.0273	0.0283	0.0298	0.0312]; 
eb_r_chg=[ 0.0235	0.0220	0.0205	0.0198	0.0198	0.0196	0.0198	0.0197	0.0203	0.0204	0.0204]; 
eb_voc=[7.2370	7.4047	7.5106	7.5873	7.6459	7.6909	7.7294	7.7666	7.8078	7.9143	8.3645]*eb_num_module;  
eb_cap_max=8.8*3600;	% (A*sec), max. capacity at 6.5 A
eb_crnt_max=8.8; % current at eb_cap_max
eb_cap_chg=8.8*3600;	% (A*sec), charging capacity
I_max_dchg=60*eb_crnt_max; % max. discharging current Ampers
I_max_chg = 40*eb_crnt_max; % max. charging current Ampers
eb_cap_time=eb_cap_max/eb_crnt_max;
% eb_min_volts=120;
% eb_max_volts=180;
eb_module_mass=0.98;  % (kg)
eb_k=1.07; % an exponent
eb_peukert_cap=(eb_crnt_max^eb_k)*eb_cap_time;
SOC_max=1;  % (--), highest limit of soc
SOC_min=0.1;  % (--), lowest limit of soc

P_max_dchg =  I_max_dchg.*eb_voc-eb_r_chg.*I_max_dchg^2;
P_max_chg = I_max_chg.*eb_voc+eb_r_chg.*I_max_chg^2;
figure(9)
plot(eb_soc,P_max_dchg/1000, eb_soc, P_max_chg/1000)
legend 'Discharge' 'Charge'
xlabel('SOC, [-]')
ylabel('Power, [kW]')
grid minor

%% VAR
load('matlab_TOOL_wOOL.mat');
SOC_init = 0.532;
SOC_delta = 0.5;
T_add = 30;


open('IVECO_Daily_P2_SIM_OOL.slx')
sim('IVECO_Daily_P2_SIM_OOL.slx')
% open('IVECO_Daily_Conv_SIM.slx')
% sim('IVECO_Daily_Conv_SIM.slx')




%% trial bsfc map
% figure(10)
% 
% contourf(w_ice_map, T_ice_map, BSFC,20,'linestyle','none')
%  colorbar
%  hold all
% contour(w_ice_map, T_ice_map, BSFC,20, 'k', 'ShowText','on');
% plot(w_ice_map, T_ice_max,'k','linewidth',1.5)
% plot(w_ice*30/pi, T_ice,'xy')
% plot(w_OOL,T_OOL,'k','linewidth',1.5)
% xlabel('Engine speed [rpm]')
% ylabel('Torque [Nm]')
% grid on
% grid minor


%%
figure(1)
T_OOL(1)=T_OOL(2);
w_OOL(1)=w_OOL(2);
contourf(w_ice_map, T_ice_map, Q_ice_map,20,'linestyle','none')
colorbar
hold all
contour(w_ice_map, T_ice_map, Q_ice_map,20, 'k', 'ShowText','on');
plot(w_ice_map, T_ice_max,'k','linewidth',1.5)
plot(w_ice*30/pi, T_ice,'oy','linewidth',0.5)
plot(w_ice_1*30/pi, T_ice_1,'xr','linewidth',0.2) %only ICE
plot(w_OOL,T_OOL,'k','linewidth',1.5)
xlabel('Engine speed [rpm]')
ylabel('Torque [Nm]')
grid on
grid minor
legend('fc','WOT','OOL','HEV','ICE')
title('WLTP')

%%%% ONLY ENGINE
% figure(7)
% contourf(w_ice_map, T_ice_map, Q_ice_map,20,'linestyle','none')
% colorbar
% hold all
% contour(w_ice_map, T_ice_map, Q_ice_map,20, 'k', 'ShowText','on');
% plot(w_ice_map, T_ice_max,'k','linewidth',1.5)
% plot(w_ice_1*30/pi, T_ice_1,'xy')
% xlabel('Engine speed [rpm]')
% ylabel('Torque [Nm]')
% grid on
% grid minor
% 


figure(2), 
plot(w_ice_map, T_ice_max,'k','linewidth',1.5)
hold all
plot(w_EM_max/Upulley, T_EM_max,'Color',[0,0.45,0.74])
plot(w_EM_max/Upulley, T_GEN_max,'Color',[0,0.45,0.74])
plot(w_OOL,T_OOL,'k','linewidth',1.5)
legend 'ICE' 'Motor'
xlabel('Angular speed at the input shaft level [rpm]'), ylabel('Torque [Nm]')
grid minor



% figure(), 
% subplot(3,1,1), plot(Time_h, E_charge/1e3,Time_h, -E_discharge/1e3)
% xlabel('Time [s]'), ylabel('Energy [kJ]')
% legend 'Charge' 'Discharge'
% grid minor
% subplot(3,1,2), plot(Time_h, (E_charge/1e3+E_discharge/1e3))
% xlabel('Time [s]'), ylabel('Energy [kJ]')
% legend 'Energy stored'
% grid minor
% subplot(3,1,3), plot(T_z, V_z)
% legend 'Speed profile'
% xlabel('Time [s]'), ylabel('Vehicle speed [m/s]')
% grid minor


% figure(4), 
% subplot(3,1,1), plot(Time_h, P_neg/1e3,Time_h, P_gen_lim/1e3)
% xlabel('Time [s]'), ylabel('Braking power [kW]')
% legend 'Brake' 'GEN Max power'
% grid minor
% xlim([0 1800])
% subplot(3,1,2), plot(Time_h, E_neg/1e3,Time_h, -E_charge/1e3)
% xlabel('Time [s]'), ylabel('Energy [kJ]')
% legend 'Braking' 'Charge' 
% grid minor
% xlim([0 1800])
% subplot(3,1,3), plot(T_z, V_z)
% legend 'Speed profile'
% xlabel('Time [s]'), ylabel('Vehicle speed [m/s]')
% grid minor
% xlim([0 1800])

figure(5), 
contour(w_em_motor, T_em_motor, eff_motor, [0:0.03:1],'ShowText','on')
hold all
contour(w_em_gen, T_em_gen, eff_gen, [0:0.03:1],'ShowText','on')
plot(w_EM_max, T_EM_max)
plot(w_GEN_max, T_GEN_max)
plot(w_EM*30/pi, T_EM,'xk')
plot(w_EM*30/pi, T_GEN,'xk')
xlabel('Angular speed [rpm]'), ylabel('Torque [Nm]')
grid minor
ylim([-120 120])

figure(6), 
plot(w_GEN_max, pi/30000*w_EM_max.*T_GEN_max) 
hold on, plot(w_GEN_max, pi/30000*w_EM_max.*T_EM_max) 
